;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F628A. This file contains the basic code              *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    16F628ATEMP.asm                                   *
;    Date:          07/2011                                           *
;    File Version:  1.2                                               *
;                                                                     *
;    Author:        Kovalev Anton YUrievich                           *
;    Company:       A                                                 *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F628A.INC                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:         Chess clock                                       *
;                                                                     *
;**********************************************************************

	list      p=16f628A           ; list directive to define processor
	#include <p16F628A.inc>       ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

    #define	BUTTON  PORTA,0
    #define	BEEP    PORTA,1
    #define	LCD_RS  PORTB,0
    #define	LCD_E   PORTB,1
    #define	LCD_DB4 PORTB,2
    #define	LCD_DB5 PORTB,3
    #define	LCD_DB6 PORTB,4
    #define	LCD_DB7 PORTB,5

;***** VARIABLE DEFINITIONS
w_temp        EQU     0x7E        ; variable used for context saving 
status_temp   EQU     0x7F        ; variable used for context saving

CONFIG_DATA  equ 0x20
PAUSE_1      equ 0x21
PAUSE_2      equ 0x22
PAUSE_3      equ 0x23
PAUSE_LCD    equ 0x24
PAUSE_BEEP   equ 0x25
LCD_BUFFER   equ 0x26
LCD_CYCLE    equ 0x27
COUNT_L      equ 0x28  
COUNT_H      equ 0x29
CLOCK_SS_W   equ 0x2A
CLOCK_MM_W   equ 0x2B 
CLOCK_CC_W   equ 0x2C
CLOCK_SS_B   equ 0x2D
CLOCK_MM_B   equ 0x2E 
CLOCK_CC_B   equ 0x2F
VL           equ 0x30
VH           equ 0x31
D0           equ 0x32 
D1           equ 0x33
D2           equ 0x34 
D3           equ 0x35
D4           equ 0x36
DD           equ 0x37  
RADR         equ 0x38
RDATA        equ 0x39

;**********************************************************************
	ORG     0x000             ; processor reset vector
	goto    main              ; go to beginning of program
	
	ORG     0x004             ; interrupt vector location
	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	status_temp       ; save off contents of STATUS register

; isr code can go here or be located as a call subroutine elsewhere

	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt

;**********************************************************************
; CONFIG_DATA 
; 0 bit - Button ON
; 1 bit - Button ON -> OFF
; 2 bit - Button pause ON
; 3 bit - WB (White or Black)
; 4 bit - New Second
; 5 bit - Pause Clock 
;**********************************************************************
main              call   Clear_RAM
                  call   EEPROM_Load_R
                  call   Ports_Initializing
                  call   TMR1_Initializing
                  call   LCD_Initializing
                  call   Pause_LCD 
                  call   LCD_Initializing
                  call   Pause_LCD
                  btfss  CONFIG_DATA,5
                  goto   General_Start
                  goto   General_Pause  
;**********************************************************************
;***************************** Clear RAM ******************************
;**********************************************************************
Clear_RAM         clrf  CONFIG_DATA  
                  clrf  PAUSE_1      
                  clrf  PAUSE_2      
                  clrf  PAUSE_3      
                  clrf  PAUSE_LCD    
                  clrf  PAUSE_BEEP   
                  clrf  LCD_BUFFER   
                  clrf  LCD_CYCLE    
                  clrf  COUNT_L       
                  clrf  COUNT_H      
                  clrf  CLOCK_SS_W   
                  clrf  CLOCK_MM_W    
                  clrf  CLOCK_CC_W   
                  clrf  CLOCK_SS_B   
                  clrf  CLOCK_MM_B    
                  clrf  CLOCK_CC_B   
                  clrf  VL           
                  clrf  VH  
                  clrf  D0            
                  clrf  D1           
                  clrf  D2            
                  clrf  D3           
                  clrf  D4           
                  clrf  DD  
                  clrf  RADR         
                  clrf  RDATA                         
                  return    
;**********************************************************************
;************************* Ports Initializing *************************
;**********************************************************************
Ports_Initializing clrf    PORTA
                   clrf    PORTB
                   clrf    CCP1CON
                   movlw   0x07
                   movwf   CMCON 
                   bsf     STATUS,RP0
                   bcf     STATUS,RP1
                   movlw   0x00
                   movwf   VRCON 
                   movlw   b'00000001'
                   movwf   TRISA  
                   movlw   b'11000000'
                   movwf   TRISB 
                   bcf     STATUS,RP0
                   return      
;**********************************************************************
;************************* TMR1 Initializing **************************
;**********************************************************************
TMR1_Initializing  clrf   T1CON
                   clrf   TMR1L
                   clrf   TMR1H
                   clrf   INTCON
                   bsf    STATUS,RP0
                   clrf   PIE1
                   bcf    STATUS,RP0
                   clrf   PIR1
                   movlw  0x3E
                   movwf  T1CON
                   bcf    T1CON,TMR1ON 
                   return
;**********************************************************************
;***************************** General Start **************************
;**********************************************************************
General_Start     clrf   TMR1L
                  clrf   TMR1H
                  clrf   CLOCK_SS_W
                  clrf   CLOCK_MM_W
                  clrf   CLOCK_CC_W
                  clrf   CLOCK_SS_B
                  clrf   CLOCK_MM_B
                  clrf   CLOCK_CC_B
                  call   LCD_WC_Clear
                  movlw  0x01
                  call   LCD_Step_Clear
                  call   LCD_LS_Rus_Start
                  call   LCD_WC_2_Line
                  movlw  0x02
                  call   LCD_Step_Clear
                  call   LCD_LS_Eng_Start
                  bcf    CONFIG_DATA,5
                  call   EEPROM_Load_W
                  call   Beep
                  call   Pause_1
                  call   Beep
                  call   Pause_2
                  call   LCD_Counter
                  call   Pause_2
                  call   LCD_Clock
                  movlw  0xF0
                  call   Button_Data_Pause
General_Start_1   call   Button                  
                  btfsc  CONFIG_DATA,1
                  goto   General_Start_2
                  btfsc  CONFIG_DATA,2
                  goto   General_Start
                  goto   General_Start_1
General_Start_2   incf   COUNT_L,f
                  movlw  0x00
                  subwf  COUNT_L,w
                  btfsc  STATUS,Z
                  incf   COUNT_H,f
                  goto   General_White
;**********************************************************************
;***************************** General White **************************
;**********************************************************************
General_White     call   Beep
                  movlw  0x40
                  call   Button_Data_Pause
                  bcf    CONFIG_DATA,3
                  bsf    T1CON,TMR1ON
General_White_1   call   Clock_Cycle 
                  btfss  CONFIG_DATA,4
                  goto   General_White_2  
                  bcf    CONFIG_DATA,4
                  call   LCD_Clock
General_White_2   call   Button                  
                  btfsc  CONFIG_DATA,1
                  goto   General_Black
                  btfsc  CONFIG_DATA,2
                  goto   General_Pause
                  goto   General_White_1                 
;**********************************************************************
;***************************** General Black **************************
;**********************************************************************
General_Black     call   Beep
                  movlw  0x40
                  call   Button_Data_Pause
                  bsf    CONFIG_DATA,3
                  bsf    T1CON,TMR1ON
General_Black_1   call   Clock_Cycle 
                  btfss  CONFIG_DATA,4
                  goto   General_Black_2  
                  bcf    CONFIG_DATA,4
                  call   LCD_Clock
General_Black_2   call   Button                  
                  btfsc  CONFIG_DATA,1
                  goto   General_White
                  btfsc  CONFIG_DATA,2
                  goto   General_Pause
                  goto   General_Black_1 
;**********************************************************************
;***************************** General Pause **************************
;**********************************************************************
General_Pause     bcf    T1CON,TMR1ON
                  bsf    CONFIG_DATA,5
                  call   EEPROM_Load_W
                  call   Beep
                  call   Pause_1 
                  call   Beep
                  movlw  0xF0
                  call   Button_Data_Pause
                  call   LCD_WC_Clear
                  movlw  0x05
                  call   LCD_Step_Clear
                  call   LCD_LS_Rus_Pause
                  call   LCD_WC_2_Line
                  movlw  0x05
                  call   LCD_Step_Clear
                  call   LCD_LS_Eng_Pause
General_Pause_2   call   Button                  
                  btfsc  CONFIG_DATA,1
                  goto   General_Pause_3
                  btfsc  CONFIG_DATA,2
                  goto   General_Start
                  goto   General_Pause_2
General_Pause_3   btfss  CONFIG_DATA,3
                  goto   General_White 
                  goto   General_Black
;**********************************************************************
;******************************** Pause 1 *****************************
;**********************************************************************
Pause_1           clrf   PAUSE_1
                  clrf   PAUSE_2
Pause_1_1         decfsz PAUSE_1,f
                  goto   Pause_1_1                
                  decfsz PAUSE_2,f
                  goto   Pause_1_1
                  return
;**********************************************************************
;******************************** Pause 2 *****************************
;**********************************************************************
Pause_2           clrf   PAUSE_1
                  clrf   PAUSE_2
                  movlw  0x0A
                  movwf  PAUSE_3
Pause_2_1         decfsz PAUSE_1,f
                  goto   Pause_2_1                
                  decfsz PAUSE_2,f
                  goto   Pause_2_1 
                  decfsz PAUSE_3,f
                  goto   Pause_2_1
                  return
;**********************************************************************
;****************************** Button ********************************
;**********************************************************************
Button            btfsc  BUTTON
                  goto   Button_1
                  btfss  BUTTON
                  goto   Button_2
                  return   
Button_1          bsf    CONFIG_DATA,0
                  decfsz PAUSE_1,f
                  return 
                  decfsz PAUSE_2,f
                  return
                  bsf    CONFIG_DATA,2   
                  return
Button_2          btfsc  CONFIG_DATA,0
                  bsf    CONFIG_DATA,1
                  return 
;**********************************************************************
;************************* Button Data Pause **************************
;**********************************************************************
Button_Data_Pause btfsc  BUTTON  
                  goto   Button_Data_Pause
                  movwf  PAUSE_2
                  clrf   PAUSE_1
                  bcf    CONFIG_DATA,0
                  bcf    CONFIG_DATA,1
                  bcf    CONFIG_DATA,2
                  return                
;**********************************************************************
;******************************** Beep ********************************
;**********************************************************************
Beep              clrf   PAUSE_1
Beep_1            bsf    BEEP
                  call   Pause_Beep 
                  bcf    BEEP
                  call   Pause_Beep 
                  decfsz PAUSE_1,f
                  goto   Beep_1
                  return    
;**********************************************************************
;***************************** Pause Beep *****************************
;**********************************************************************
Pause_Beep        movlw  0x27
                  movwf  PAUSE_BEEP
Pause_Beep_1      decfsz PAUSE_BEEP,f
                  goto   Pause_Beep_1
                  return
;**********************************************************************
;**************************** Clock Cycle *****************************
;**********************************************************************
Clock_Cycle       movlw  0x0F
                  subwf  TMR1H,w
                  btfsc  STATUS,C
                  return
                  movlw  0x10
                  subwf  TMR1H,f
                  bsf    CONFIG_DATA,4
                  btfsc  CONFIG_DATA,3
                  goto   Clock_Cycle_B 
;**********************************************************************
Clock_Cycle_W     incf   CLOCK_SS_W,f
                  movlw  0x3C
                  subwf  CLOCK_SS_W,w
                  btfss  STATUS,Z  
                  goto   Clock_Cycle_W_1
                  clrf   CLOCK_SS_W
                  incf   CLOCK_MM_W,f
Clock_Cycle_W_1   movlw  0x3C
                  subwf  CLOCK_MM_W,w
                  btfss  STATUS,Z  
                  goto   Clock_Cycle_W_2
                  clrf   CLOCK_MM_W
                  incf   CLOCK_CC_W,f
Clock_Cycle_W_2   movlw  0x18
                  subwf  CLOCK_CC_W,w
                  btfss  STATUS,Z  
                  return
                  clrf   CLOCK_CC_W                 
                  return
;**********************************************************************
Clock_Cycle_B     incf   CLOCK_SS_B,f
                  movlw  0x3C
                  subwf  CLOCK_SS_B,w
                  btfss  STATUS,Z  
                  goto   Clock_Cycle_B_1
                  clrf   CLOCK_SS_B
                  incf   CLOCK_MM_B,f
Clock_Cycle_B_1   movlw  0x3C
                  subwf  CLOCK_MM_B,w
                  btfss  STATUS,Z  
                  goto   Clock_Cycle_B_2
                  clrf   CLOCK_MM_B
                  incf   CLOCK_CC_B,f
Clock_Cycle_B_2   movlw  0x18
                  subwf  CLOCK_CC_B,w
                  btfss  STATUS,Z  
                  return
                  clrf   CLOCK_CC_B                 
                  return
;**********************************************************************
;***************************** LCD Clock ******************************
;**********************************************************************
LCD_Clock         call   LCD_WC_Clear
                  call   LCD_Symbol_Rus_B
                  movlw  0x03
                  call   LCD_Step_Clear
                  clrf   VH
                  movf   CLOCK_CC_W,w
                  movwf  VL
                  call   Convert_VHL  
                  call   W_LCD_D2L
                  call   LCD_Symbol_..
                  movf   CLOCK_MM_W,w
                  movwf  VL
                  call   Convert_VHL  
                  call   W_LCD_D2L
                  call   LCD_Symbol_..
                  movf   CLOCK_SS_W,w
                  movwf  VL
                  call   Convert_VHL  
                  call   W_LCD_D2L
                  movlw  0x03
                  call   LCD_Step_Clear
                  call   LCD_Symbol_Eng_W
;**********************************************************************
                  call   LCD_WC_2_Line
                  call   LCD_Symbol_Rus_CH
                  movlw  0x03
                  call   LCD_Step_Clear
                  clrf   VH
                  movf   CLOCK_CC_B,w
                  movwf  VL
                  call   Convert_VHL  
                  call   W_LCD_D2L
                  call   LCD_Symbol_..
                  movf   CLOCK_MM_B,w
                  movwf  VL
                  call   Convert_VHL  
                  call   W_LCD_D2L
                  call   LCD_Symbol_..
                  movf   CLOCK_SS_B,w
                  movwf  VL
                  call   Convert_VHL  
                  call   W_LCD_D2L
                  movlw  0x03
                  call   LCD_Step_Clear
                  call   LCD_Symbol_Eng_B
                  return
;**********************************************************************
;**************************** LCD Counter *****************************
;**********************************************************************
LCD_Counter       call   LCD_WC_Clear
                  movlw  0x05
                  call   LCD_Step_Clear
                  movf   COUNT_H,w
                  movwf  VH
                  movf   COUNT_L,w
                  movwf  VL
                  call   Convert_VHL 
                  call   W_LCD_DH  
                  call   W_LCD_DL
                  return
;**********************************************************************
;************ Convert VH_VL - D4_D3_D2_D1_D0 Write_DD_LCD *************
;**********************************************************************
Convert_VHL clrf  D4
            clrf  D3
            clrf  D2
            clrf  D1
            clrf  D0
;**********************************************************************
Bit0        btfss VL,0
            goto  Bit1
            incf  D0,f
Bit1        btfss VL,1
            goto  Bit2 
            movlw 0x02
            addwf D0,f 
Bit2        btfss VL,2
            goto  Bit3 
            movlw 0x04
            addwf D0,f
Bit3        btfss VL,3
            goto  Bit4 
            movlw 0x08
            addwf D0,f
Bit4        btfss VL,4
            goto  Bit5 
            movlw 0x06
            addwf D0,f
            incf  D1,f
Bit5        btfss VL,5
            goto  Bit6 
            movlw 0x02
            addwf D0,f
            movlw 0x03
            addwf D1,f
Bit6        btfss VL,6
            goto  Bit7 
            movlw 0x04
            addwf D0,f
            movlw 0x06
            addwf D1,f
Bit7        btfss VL,7
            goto  Bit8 
            movlw 0x08
            addwf D0,f
            movlw 0x02
            addwf D1,f
            incf  D2,f
;**********************************************************************
Bit8        btfss VH,0
            goto  Bit9 
            movlw 0x06
            addwf D0,f
            movlw 0x05
            addwf D1,f
            movlw 0x02
            addwf D2,f
Bit9        btfss VH,1
            goto  Bit10 
            movlw 0x02
            addwf D0,f
            incf  D1,f
            movlw 0x05
            addwf D2,f
Bit10       btfss VH,2
            goto  Bit11 
            movlw 0x04
            addwf D0,f
            movlw 0x02
            addwf D1,f
            incf  D3,f
Bit11       btfss VH,3
            goto  Bit12 
            movlw 0x08
            addwf D0,f
            movlw 0x04
            addwf D1,f
            movlw 0x02
            addwf D3,f
Bit12       btfss VH,4
            goto  Bit13 
            movlw 0x06
            addwf D0,f
            movlw 0x09
            addwf D1,f
            movlw 0x04
            addwf D3,f
Bit13       btfss VH,5
            goto  Bit14 
            movlw 0x02
            addwf D0,f
            movlw 0x09
            addwf D1,f
            incf  D2,f
            movlw 0x08
            addwf D3,f
Bit14       btfss VH,6
            goto  Bit15 
            movlw 0x04
            addwf D0,f
            movlw 0x08
            addwf D1,f
            movlw 0x03
            addwf D2,f
            movlw 0x06
            addwf D3,f
            incf  D4,f
Bit15       btfss VH,7
            goto  D0_ready 
            movlw 0x08
            addwf D0,f
            movlw 0x06
            addwf D1,f
            movlw 0x07
            addwf D2,f
            movlw 0x02
            addwf D3,f
            movlw 0x03
            addwf D4,f
;**********************************************************************
D0_ready    movlw 0x0A
            subwf D0,W
            btfss STATUS,C
            goto  D1_ready
            movwf D0
            incf  D1,f
            goto  D0_ready
D1_ready    movlw 0x0A
            subwf D1,W
            btfss STATUS,C
            goto  D2_ready
            movwf D1
            incf  D2,f
            goto  D1_ready          
D2_ready    movlw 0x0A
            subwf D2,W
            btfss STATUS,C
            goto  D3_ready
            movwf D2
            incf  D3,f
            goto  D2_ready  
D3_ready    movlw 0x0A
            subwf D3,W
            btfss STATUS,C
            goto  D4_ready
            movwf D3
            incf  D4,f
            goto  D3_ready 
D4_ready    return 
;**********************************************************************
;********************** Write D-code for LCD **************************
;**********************************************************************
W_LCD_DH    movf  D4,w
            movwf DD
            call  D_Symbol_LCD 
            movf  D3,w
            movwf DD 
            call  D_Symbol_LCD 
            return 
W_LCD_DL    movf  D2,w
            movwf DD 
            call  D_Symbol_LCD 
W_LCD_D2L   movf  D1,w
            movwf DD 
            call  D_Symbol_LCD 
            movf  D0,w
            movwf DD 
            call  D_Symbol_LCD
            return
;**********************************************************************
;************************* D-code for LCD *****************************
;**********************************************************************
D_Symbol_LCD    movlw 0x00
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_0
                movlw 0x01
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_1
                movlw 0x02
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_2
                movlw 0x03
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_3
                movlw 0x04
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_4
                movlw 0x05
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_5
                movlw 0x06
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_6
                movlw 0x07
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_7
                movlw 0x08
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_8
                movlw 0x09
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_9
                return
;**********************************************************************
;******************************* LCD **********************************
;**********************************************************************
LCD_PORTB_Write_L bcf    LCD_DB4
                  bcf    LCD_DB5
                  bcf    LCD_DB6
                  bcf    LCD_DB7
                  btfsc  LCD_BUFFER,0
                  bsf    LCD_DB4  
                  btfsc  LCD_BUFFER,1
                  bsf    LCD_DB5
                  btfsc  LCD_BUFFER,2
                  bsf    LCD_DB6
                  btfsc  LCD_BUFFER,3
                  bsf    LCD_DB7   
                  return   
;**********************************************************************
LCD_PORTB_Write_H bcf    LCD_DB4
                  bcf    LCD_DB5
                  bcf    LCD_DB6
                  bcf    LCD_DB7
                  btfsc  LCD_BUFFER,4
                  bsf    LCD_DB4  
                  btfsc  LCD_BUFFER,5
                  bsf    LCD_DB5
                  btfsc  LCD_BUFFER,6
                  bsf    LCD_DB6
                  btfsc  LCD_BUFFER,7
                  bsf    LCD_DB7   
                  return   
;**********************************************************************
LCD_WC            movwf  LCD_BUFFER
                  bcf    LCD_RS
                  call   LCD_PORTB_Write_H
                  bsf    LCD_E
                  call   Pause_LCD 
                  bcf    LCD_E 
                  call   Pause_LCD
                  return
;**********************************************************************
LCD_WD            movwf  LCD_BUFFER
                  bsf    LCD_RS
                  call   LCD_PORTB_Write_H
                  bsf    LCD_E  
                  nop
                  nop
                  bcf    LCD_E   
                  call   Pause_LCD
                  call   LCD_PORTB_Write_L
                  bsf    LCD_E   
                  nop
                  nop                 
                  bcf    LCD_E 
                  call   Pause_LCD
                  return
;**********************************************************************
LCD_Initializing  movlw  b'00110000'
                  call   LCD_WC
                  call   Pause_LCD
                  movlw  b'00100000'
                  call   LCD_WC
                  movlw  b'10000000'
                  call   LCD_WC            
                  movlw  b'00100000'
                  call   LCD_WC
                  movlw  b'10000000'
                  call   LCD_WC
                  movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'11000000'
                  call   LCD_WC
                  movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'00010000'
                  call   LCD_WC
                  call   Pause_LCD
                  movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'01100000'
                  call   LCD_WC
                  return
;**********************************************************************
LCD_WC_Clear      movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'00010000'
                  call   LCD_WC
                  call   Pause_LCD
                  return
;**********************************************************************
LCD_Step_Clear    movwf  LCD_CYCLE
LCD_Step_Clear_1  call   LCD_Symbol_
                  decfsz LCD_CYCLE,f
                  goto   LCD_Step_Clear_1 
                  return
;**********************************************************************
LCD_WC_2_Line     bcf   LCD_RS
                  movlw b'10110000'
                  call  LCD_WC
                  movlw b'10110000'
                  call  LCD_WC
                  movlw 0x05
                  call  LCD_Step_Clear
                  return
;**********************************************************************
;***************************** Pause LCD ******************************
;**********************************************************************
Pause_LCD        clrf    PAUSE_LCD
Pause_LCD_1      decfsz  PAUSE_LCD,f
                 goto    Pause_LCD_1
                 return   
;**********************************************************************
;**************************** LCD Symbol ******************************
;**********************************************************************
LCD_Symbol_Eng_A  movlw  b'01000001' ;A
                  call   LCD_WD
                  return
LCD_Symbol_Eng_a  movlw  b'01100001' ;a
                  call   LCD_WD
                  return
LCD_Symbol_Eng_B  movlw  b'01000010' ;B
                  call   LCD_WD
                  return
LCD_Symbol_Eng_b  movlw  b'01100010' ;b
                  call   LCD_WD
                  return
LCD_Symbol_Eng_C  movlw  b'01000011' ;C
                  call   LCD_WD 
                  return
LCD_Symbol_Eng_c  movlw  b'01100011' ;c
                  call   LCD_WD 
                  return
LCD_Symbol_Eng_D  movlw  b'01000100' ;D
                  call   LCD_WD
                  return
LCD_Symbol_Eng_d  movlw  b'01100100' ;d
                  call   LCD_WD
                  return
LCD_Symbol_Eng_E  movlw  b'01000101' ;E
                  call   LCD_WD
                  return
LCD_Symbol_Eng_e  movlw  b'01100101' ;e
                  call   LCD_WD
                  return
LCD_Symbol_Eng_F  movlw  b'01000110' ;F
                  call   LCD_WD
                  return
LCD_Symbol_Eng_f  movlw  b'01100110' ;f
                  call   LCD_WD
                  return
LCD_Symbol_Eng_G  movlw  b'01000111' ;G
                  call   LCD_WD
                  return
LCD_Symbol_Eng_g  movlw  b'01100111' ;g
                  call   LCD_WD
                  return
LCD_Symbol_Eng_H  movlw  b'01001000' ;H
                  call   LCD_WD
                  return
LCD_Symbol_Eng_h  movlw  b'01101000' ;h
                  call   LCD_WD
                  return
LCD_Symbol_Eng_I  movlw  b'01001001' ;I
                  call   LCD_WD
                  return
LCD_Symbol_Eng_i  movlw  b'01101001' ;i
                  call   LCD_WD
                  return
LCD_Symbol_Eng_J  movlw  b'01001010' ;J
                  call   LCD_WD
                  return
LCD_Symbol_Eng_j  movlw  b'01101010' ;j
                  call   LCD_WD
                  return
LCD_Symbol_Eng_K  movlw  b'01001011' ;K
                  call   LCD_WD
                  return
LCD_Symbol_Eng_k  movlw  b'01101011' ;k
                  call   LCD_WD
                  return
LCD_Symbol_Eng_L  movlw  b'01001100' ;L
                  call   LCD_WD
                  return
LCD_Symbol_Eng_l  movlw  b'01101100' ;l
                  call   LCD_WD
                  return
LCD_Symbol_Eng_M  movlw  b'01001101' ;M
                  call   LCD_WD
                  return
LCD_Symbol_Eng_m  movlw  b'01101101' ;m
                  call   LCD_WD
                  return
LCD_Symbol_Eng_N  movlw  b'01001110' ;N
                  call   LCD_WD
                  return
LCD_Symbol_Eng_n  movlw  b'01101110' ;n
                  call   LCD_WD
                  return
LCD_Symbol_Eng_O  movlw  b'01001111' ;O
                  call   LCD_WD
                  return
LCD_Symbol_Eng_o  movlw  b'01101111' ;o
                  call   LCD_WD
                  return
LCD_Symbol_Eng_P  movlw  b'01010000' ;P
                  call   LCD_WD
                  return
LCD_Symbol_Eng_p  movlw  b'01110000' ;p
                  call   LCD_WD
                  return
LCD_Symbol_Eng_Q  movlw  b'01010001' ;Q
                  call   LCD_WD
                  return
LCD_Symbol_Eng_q  movlw  b'01110001' ;q
                  call   LCD_WD
                  return
LCD_Symbol_Eng_R  movlw  b'01010010' ;R
                  call   LCD_WD
                  return
LCD_Symbol_Eng_r  movlw  b'01110010' ;r
                  call   LCD_WD
                  return
LCD_Symbol_Eng_S  movlw  b'01010011' ;S
                  call   LCD_WD
                  return
LCD_Symbol_Eng_s  movlw  b'01110011' ;s
                  call   LCD_WD
                  return
LCD_Symbol_Eng_T  movlw  b'01010100' ;T
                  call   LCD_WD
                  return
LCD_Symbol_Eng_t  movlw  b'01110100' ;t
                  call   LCD_WD
                  return
LCD_Symbol_Eng_U  movlw  b'01010101' ;U
                  call   LCD_WD
                  return
LCD_Symbol_Eng_u  movlw  b'01110101' ;u
                  call   LCD_WD
                  return
LCD_Symbol_Eng_V  movlw  b'01010110' ;V
                  call   LCD_WD
                  return
LCD_Symbol_Eng_v  movlw  b'01110110' ;v
                  call   LCD_WD
                  return
LCD_Symbol_Eng_W  movlw  b'01010111' ;W
                  call   LCD_WD
                  return
LCD_Symbol_Eng_w  movlw  b'01110111' ;w
                  call   LCD_WD
                  return
LCD_Symbol_Eng_X  movlw  b'01011000' ;X
                  call   LCD_WD
                  return
LCD_Symbol_Eng_x  movlw  b'01111000' ;x
                  call   LCD_WD
                  return
LCD_Symbol_Eng_Y  movlw  b'01011001' ;Y
                  call   LCD_WD
                  return
LCD_Symbol_Eng_y  movlw  b'01111001' ;y
                  call   LCD_WD
                  return
LCD_Symbol_Eng_Z  movlw  b'01011010' ;Z
                  call   LCD_WD
                  return
LCD_Symbol_Eng_z  movlw  b'01111010' ;z
                  call   LCD_WD
                  return
;**********************************************************************
LCD_Symbol_Rus_A  movlw  b'01000001' ;A
                  call   LCD_WD
                  return
LCD_Symbol_Rus_a  movlw  b'01100001' ;a
                  call   LCD_WD
                  return
LCD_Symbol_Rus_B  movlw  b'10100000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_b  movlw  b'10110010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_V  movlw  b'01000010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_v  movlw  b'10110011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_G  movlw  b'10100001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_g  movlw  b'10110100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_D  movlw  b'11100000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_d  movlw  b'11100011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_E  movlw  b'01000101' ;E
                  call   LCD_WD
                  return
LCD_Symbol_Rus_e  movlw  b'01100101' ;e
                  call   LCD_WD
                  return
LCD_Symbol_Rus_YO movlw  b'10100010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_yo movlw  b'10110101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_J  movlw  b'10100011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_j  movlw  b'10110110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_Z  movlw  b'10100100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_z  movlw  b'10110111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_I  movlw  b'10100101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_i  movlw  b'10111000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_IK movlw  b'10100110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ik movlw  b'10111001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_K  movlw  b'01001011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_k  movlw  b'10111010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_L  movlw  b'10100111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_l  movlw  b'10111011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_M  movlw  b'01001101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_m  movlw  b'10111100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_N  movlw  b'01001000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_n  movlw  b'10111101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_O  movlw  b'01001111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_o  movlw  b'01101111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_P  movlw  b'10101000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_p  movlw  b'10111110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_R  movlw  b'01010000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_r  movlw  b'01110000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_S  movlw  b'01000011' ;
                  call   LCD_WD 
                  return
LCD_Symbol_Rus_s  movlw  b'01100011' ;
                  call   LCD_WD 
                  return
LCD_Symbol_Rus_T  movlw  b'01010100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_t  movlw  b'10111111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_U  movlw  b'10101001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_u  movlw  b'01111001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_F  movlw  b'10101010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_f  movlw  b'11100100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_H  movlw  b'01011000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_h  movlw  b'01111000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_C  movlw  b'11100001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_c  movlw  b'11100101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_CH movlw  b'10101011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ch movlw  b'11000000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_SH movlw  b'10101100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_sh movlw  b'11000001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_SHA movlw  b'11100010' ;
                   call   LCD_WD
                   return
LCD_Symbol_Rus_sha movlw  b'11100110' ;
                   call   LCD_WD
                   return
LCD_Symbol_Rus_TZ movlw  b'10101101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_tz movlw  b'11000010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_Y  movlw  b'10101110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_y  movlw  b'11000011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_MZ movlw  b'01100010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_mz movlw  b'11000100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_EE movlw  b'10101111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ee movlw  b'11000101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_YU movlw  b'10110000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_yu movlw  b'11000110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_YA movlw  b'10110001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ya movlw  b'11000111' ;
                  call   LCD_WD
                  return
;**********************************************************************
LCD_Symbol_0      movlw  b'00110000' ;0
                  call   LCD_WD 
                  return
LCD_Symbol_1      movlw  b'00110001' ;1
                  call   LCD_WD 
                  return
LCD_Symbol_2      movlw  b'00110010' ;2
                  call   LCD_WD 
                  return
LCD_Symbol_3      movlw  b'00110011' ;3
                  call   LCD_WD 
                  return
LCD_Symbol_4      movlw  b'00110100' ;4
                  call   LCD_WD 
                  return
LCD_Symbol_5      movlw  b'00110101' ;5
                  call   LCD_WD 
                  return
LCD_Symbol_6      movlw  b'00110110' ;6
                  call   LCD_WD 
                  return
LCD_Symbol_7      movlw  b'00110111' ;7
                  call   LCD_WD 
                  return
LCD_Symbol_8      movlw  b'00111000' ;8
                  call   LCD_WD 
                  return
LCD_Symbol_9      movlw  b'00111001' ;9
                  call   LCD_WD 
                  return
;**********************************************************************
LCD_Symbol_       movlw  b'00010000' ;
                  call   LCD_WD
                  return
LCD_Symbol_.      movlw  b'00101110' ;.
                  call   LCD_WD 
                  return
LCD_Symbol_..     movlw  b'00111010' ;:
                  call   LCD_WD 
                  return
LCD_Symbol_scb    movlw  b'00101000' ;(
                  call   LCD_WD 
                  return
LCD_Symbol_sce    movlw  b'00101001' ;)
                  call   LCD_WD 
                  return
LCD_Symbol_spb    movlw  b'01011011' ;[
                  call   LCD_WD 
                  return
LCD_Symbol_spe    movlw  b'01011101' ;]
                  call   LCD_WD
                  return
;**********************************************************************
;************************ LCD Link Symbols ****************************
;**********************************************************************
LCD_LS_Rus_Start  call  LCD_Symbol_Rus_SH
                  call  LCD_Symbol_Rus_a
                  call  LCD_Symbol_Rus_h
                  call  LCD_Symbol_Rus_m
                  call  LCD_Symbol_Rus_a
                  call  LCD_Symbol_Rus_t
                  call  LCD_Symbol_Rus_n
                  call  LCD_Symbol_Rus_y
                  call  LCD_Symbol_Rus_e
                  call  LCD_Symbol_
                  call  LCD_Symbol_Rus_ch
                  call  LCD_Symbol_Rus_a
                  call  LCD_Symbol_Rus_s
                  call  LCD_Symbol_Rus_y
                  return         
;**********************************************************************
LCD_LS_Eng_Start  call  LCD_Symbol_Eng_C
                  call  LCD_Symbol_Eng_h
                  call  LCD_Symbol_Eng_e
                  call  LCD_Symbol_Eng_s
                  call  LCD_Symbol_Eng_s
                  call  LCD_Symbol_
                  call  LCD_Symbol_Eng_c
                  call  LCD_Symbol_Eng_l
                  call  LCD_Symbol_Eng_o
                  call  LCD_Symbol_Eng_c
                  call  LCD_Symbol_Eng_k
                  return 
;**********************************************************************
LCD_LS_Rus_Pause  call  LCD_Symbol_Rus_P
                  call  LCD_Symbol_Rus_a
                  call  LCD_Symbol_Rus_u
                  call  LCD_Symbol_Rus_z
                  call  LCD_Symbol_Rus_a
                  return
;**********************************************************************
LCD_LS_Eng_Pause  call  LCD_Symbol_Eng_P
                  call  LCD_Symbol_Eng_a
                  call  LCD_Symbol_Eng_u
                  call  LCD_Symbol_Eng_s
                  call  LCD_Symbol_Eng_e
                  return
;********************************************************************** 
;******************************* EEPROM *******************************
;********************************************************************** 
; 0x00 - CONFIG_DATA
; 0x01 - COUNT_L        
; 0x02 - COUNT_H      
; 0x03 - TMR1L
; 0x04 - TMR1H
; 0x05 - CLOCK_SS_W  
; 0x06 - CLOCK_MM_W  
; 0x07 - CLOCK_CC_W   
; 0x08 - CLOCK_SS_B   
; 0x09 - CLOCK_MM_B   
; 0x0A - CLOCK_CC_B   
;********************************************************************** 
EEPROM_W         movf    RDATA,W
                 bsf     STATUS,RP0 
                 bsf     EECON1,WREN 
                 movwf   EEDATA 
                 bcf     STATUS,RP0
                 movf    RADR,W
                 bsf     STATUS,RP0 
                 movwf   EEADR
                 bcf     INTCON,GIE
                 movlw   0x55
                 movwf   EECON2
                 movlw   0xAA
                 movwf   EECON2
                 bsf     EECON1,WR
                 bsf     INTCON,GIE
                 bcf     STATUS,RP0
                 call    EEPROM_R           
                 subwf   RDATA,W
                 btfss   STATUS,Z   
                 goto    EEPROM_W
                 return
;**********************************************************************
EEPROM_R         movf    RADR,W
                 bsf     STATUS,RP0
                 bcf     STATUS,RP1
                 movwf   EEADR
                 bsf     EECON1,RD
                 movf    EEDATA,W 
                 bcf     STATUS,RP0
                 return 
;********************************************************************** 
;*************************** EEPROM Load R ****************************
;********************************************************************** 
EEPROM_Load_R    movlw   0x00
                 movwf   RADR
                 call    EEPROM_R
                 movwf   CONFIG_DATA 
                 movlw   0x01
                 movwf   RADR
                 call    EEPROM_R
                 movwf   COUNT_L  
                 movlw   0x02
                 movwf   RADR
                 call    EEPROM_R
                 movwf   COUNT_H
                 movlw   0x03
                 movwf   RADR
                 call    EEPROM_R
                 movwf   TMR1L
                 movlw   0x04
                 movwf   RADR
                 call    EEPROM_R
                 movwf   TMR1H
                 movlw   0x05
                 movwf   RADR
                 call    EEPROM_R
                 movwf   CLOCK_SS_W
                 movlw   0x06
                 movwf   RADR
                 call    EEPROM_R
                 movwf   CLOCK_MM_W
                 movlw   0x07
                 movwf   RADR
                 call    EEPROM_R
                 movwf   CLOCK_CC_W
                 movlw   0x08
                 movwf   RADR
                 call    EEPROM_R
                 movwf   CLOCK_SS_B
                 movlw   0x09
                 movwf   RADR
                 call    EEPROM_R
                 movwf   CLOCK_MM_B
                 movlw   0x0A
                 movwf   RADR
                 call    EEPROM_R
                 movwf   CLOCK_CC_B
                 return
;********************************************************************** 
;*************************** EEPROM Load W ****************************
;********************************************************************** 
EEPROM_Load_W    movlw   0x00
                 movwf   RADR
                 movf    CONFIG_DATA,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x01
                 movwf   RADR
                 movf    COUNT_L,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x02
                 movwf   RADR
                 movf    COUNT_H,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x03
                 movwf   RADR
                 movf    TMR1L,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x04
                 movwf   RADR
                 movf    TMR1H,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x05
                 movwf   RADR
                 movf    CLOCK_SS_W,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x06
                 movwf   RADR
                 movf    CLOCK_MM_W,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x07
                 movwf   RADR
                 movf    CLOCK_CC_W,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x08
                 movwf   RADR
                 movf    CLOCK_SS_B,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x09
                 movwf   RADR
                 movf    CLOCK_MM_B,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x0A
                 movwf   RADR
                 movf    CLOCK_CC_B,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 return
;**********************************************************************

; initialize eeprom locations

	ORG	0x2100
	DE	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00


	END                       ; directive 'end of program'

